// -*-Mode: C++;-*-

// Grin LISP
// Copyright (C) 2001 Daniel Beer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

namespace lisp {
  class cons : public variable {
  public:
    weak_object car, cdr;

    cons(void) { }
    cons(object a, object d) : car(a), cdr(d) { }

    virtual string print(void) const;
    virtual bool equal(object) const;
    virtual int recursive_mark(void);
    virtual const char *get_typeof(void) const;
  };

  inline cons& assume_cons(object o) { return *(cons *)(o.get_data()); }

  inline bool is_cons(const object o) {
    return dynamic_cast<const cons *>(o.get_const_data())!=0;
  }

  inline cons& as_cons(object o) {
    cons *c=dynamic_cast<cons *>(o.get_data());
    if(c) return *c;
    throw error(o, _("Type mismatch"));
  }
};
